function reset()
	rope.read("reset.lua")
end

function love.load()
	math.randomseed(os.time())	
	require "ow_rope"
	require "ow_illusion"
	require "ow_control"
	
	logo = love.graphics.newImage("logo.png")
	font = love.graphics.newFont("Lekton-Bold.ttf",14)
	debugfont = love.graphics.newFont( "Lekton-Bold.ttf",12)
	love.graphics.setFont(font)
	love.graphics.setLineStyle("rough")
	projects = recursiveEnumerate("projects")
	
	table.sort(projects, function (a,b) return (a < b) end)
	colours = {{math.random(0,100),math.random(0,100),math.random(0,100)},}

	currenttable = pretable
	projectselected = {"null","null"}
	projectviewmode = "details"
	typedinput = false
	debug = false
	sliderspeed = 100
	sliderinfo = {{225,math.random(0,100),sliderspeed},{225,math.random(0,100),sliderspeed},{225,math.random(0,100),sliderspeed},{255,255,sliderspeed*2}}
	
	focusproject = "DeerHunter1066"
	rope.read("projects/"..focusproject.."/info.lua")
	if info.image ~= nil then
		focusimage = info.image
	end
	if focusproject ~= nil then
		table.remove(projects,whereintable(focusproject,projects))
		table.insert(projects,1,focusproject)
	end	
end

function love.update(dt)
	mX,mY = love.mouse.getX(),love.mouse.getY()
	
	updater(dt)
end

function updater(dt)
	slider(dt)
end

function love.draw()
	graphics()
	
	if debug then
		if currenttable ~= null then
			love.graphics.setColor(200,200,200,100)
			for i = 1,#currenttable do
				love.graphics.rectangle("fill",currenttable[i].x,currenttable[i].y,currenttable[i].w,currenttable[i].h)
			end
			love.graphics.setColor(255,255,255,255)
		end
	
		love.graphics.setFont(debugfont)
		love.graphics.setColor(0,0,0,100)
		if mX + 125 > love.graphics.getWidth() then
			love.graphics.rectangle("fill",mX-120,mY+5,120,50)
			love.graphics.setColor(255,255,255,255)
			love.graphics.print("Mpos="..love.mouse.getX().." : "..love.mouse.getY(),mX-115,mY+5)
			love.graphics.print("fps="..love.timer.getFPS(),mX-115,mY+17)
		else
			love.graphics.rectangle("fill",mX+5,mY+5,120,50)
			love.graphics.setColor(255,255,255,255)
			love.graphics.print("Mpos="..love.mouse.getX().." : "..love.mouse.getY(),mX+10,mY+5)
			love.graphics.print("fps="..love.timer.getFPS(),mX+10,mY+17)
		end
		love.graphics.setFont(font)
	end
end

function graphics()
	love.graphics.setColor(sliderinfo[1][1],sliderinfo[2][1],sliderinfo[3][1])
	love.graphics.rectangle("fill",0,0,250,400)
	love.graphics.setColor(255,255,255)
	love.graphics.draw(logo,5,5,0,0.5)
	love.graphics.setColor(255,255,255,sliderinfo[4][1])
	-- if in menu then draw menu
	if sliderinfo[4][1] > 0 then
		-- if no focusproject then
		if focusproject == nil then
			love.graphics.print("Games & Concepts:",10,100)
			for i,v in ipairs(projects) do
				love.graphics.rectangle("fill",10,103+(i*14),7,7)
				love.graphics.print(v,20,100+(i*14))
			end
		
		-- if focusproject then display special frontpage
		else
			love.graphics.setColor(255,255,255,sliderinfo[4][1])
			love.graphics.print("Games & Concepts:",10,100)
			if focusimage ~= nil then
				love.graphics.draw(focusimage,5,115)
			else
				love.graphics.rectangle("fill",5,115,240,150)
			end
			love.graphics.setColor(sliderinfo[1][1],sliderinfo[2][1],sliderinfo[3][1],sliderinfo[4][1])
			love.graphics.rectangle("fill",5,245,25+font:getWidth(focusproject),20)
			love.graphics.setColor(255,255,255,sliderinfo[4][1])
			love.graphics.line(30+font:getWidth(focusproject),265,245,265,245,115,5,115,5,245,30+font:getWidth(focusproject),245,30+font:getWidth(focusproject),265)
			for i,v in ipairs(projects) do
				love.graphics.rectangle("fill",10,240+(i*14),7,7)
				love.graphics.print(v,20,237+(i*14))
			end
		end
		
	-- if in game selection then draw that
	else
		love.graphics.setColor(255,255,255,sliderinfo[5][1])
		love.graphics.print(projectselected[2],10,100)
		
		if info.image ~= nil then
			love.graphics.draw(info.image,5,115)
		else
			love.graphics.rectangle("fill",5,115,240,150)
		end
		
		love.graphics.setColor(sliderinfo[1][1],sliderinfo[2][1],sliderinfo[3][1],sliderinfo[5][1])
		love.graphics.rectangle("fill",5,245,20,20)
		love.graphics.setColor(255,255,255,sliderinfo[5][1])
		-- if show details
		if projectviewmode == "details" then
			love.graphics.circle("fill",15,255,5,10)
			if info.description ~= nil then
				love.graphics.printf(info.description,5,270,240,"left")
			end
			
		-- if show settings
		else
			love.graphics.rectangle("fill",10,250,10,10)
			for i,v in ipairs(info.settings) do
				love.graphics.print(v[1],5,270+(28*(i-1)))
			end
		end
		love.graphics.line(25,265,245,265,245,115,5,115,5,245,25,245,25,265)
		
		-- reject/accept
		love.graphics.print("Return.",10,376)
		if love.filesystem.exists("projects/"..projectselected[1].."/main.lua") then
			love.graphics.print("Play.",205,376)
		end
	end
end