-- HOLDS ALL GAME CONTROLS AND SOME FUNCTIONS RELATING TO THEM

-- interaction tables used by the menu system
pretable = {
	{x=5,y=5,w=240,h=95,action={"pre","companyinfo"},},
	{x=5,y=115,w=240,h=280,action={"pre","projectselect"},},
}

-- currenttable to draw menu style interactions from
-- now in startup code
currenttable = currenttable

-- click area detection functions for menu code
function nearestmenu(x,y)
	current = nearestestui(x,y,currenttable)
	return current or null
end

function nearestestui(x,y,table)
	for i,v in ipairs(table) do
		if checkCollision(x,y,1,1,v.x,v.y,v.w,v.h) == true then
			id = i
		end
	end
	return id
end

--input library for mousepresses
function love.mousepressed(x,y,button)
	if button == "l" then
		typedinput = false
		mouseAction(x,y,1)
	end
	if button == "r" then
		typedinput = false
		mouseAction(x,y,2)
	end
	if button == "wu" then
		mouseAction(x,y,3)
	end
	if button == "wd" then
		mouseAction(x,y,4)
	end
	if button == "m" then
		mouseAction(x,y,5)
	end
end

function love.mousereleased(x,y,button)
	if button == "m" then
		mouseAction(x,y,5)
	end
end

-- HUD DETECTION CODE


--SUPERFUNCTION
--controls all of the mouse actions within the editor
--a bit cryptic but not to hard to understand
function mouseAction(x,y,act)
	----MENU
	local n = nearestmenu(x,y)
	
	--NULL INPUT
	--stops it from crashing if theres nothing at click
	if n ~= null then
		if currenttable[n].action[1] == "pre" then
			-- the only input in this area, selects and controls the settings for projects
			if currenttable[n].action[2] == "companyinfo" then
				if projectselected[1] == "null" then
					projectselected = {"About ObscureWorlds","About ObscureWorlds"}
					info = {
						image = love.graphics.newImage("preview.png"),
						description = "Obscure Worlds aims for the unusual and unique, but really I just end up having fun and making what I want. Contact me here obscureworlds@gmail.com. I also do websites!",
					}
					for i = 1,3 do
						sliderinfo[i][2] = math.random(0,100)
					end
					sliderinfo[5] = {-255,255,sliderspeed*2}
					sliderinfo[4] = {255,0,sliderspeed*2}
				end
			elseif currenttable[n].action[2] == "projectselect" then
				-- check if a project has already been selected, if yes then continue to else
				if projectselected[1] == "null" then
					local focusoffset = 0
					if focusproject ~= nil then
						focusoffset = 137
					end
					for i = 1,#projects do
						if focusproject ~= nil and i == 1 and y >= currenttable[n].y and y <= currenttable[n].y + 150 then
							projectselected = {focusproject,focusproject}
							rope.read("projects/"..projectselected[1].."/info.lua")
							for i = 1,3 do
								sliderinfo[i][2] = math.random(0,100)
							end
							sliderinfo[5] = {-255,255,sliderspeed*2}
							sliderinfo[4] = {255,0,sliderspeed*2}
						else
							if y >= currenttable[n].y + focusoffset + (14*(i-1)) and y <= currenttable[n].y + focusoffset + (14*i) then
								projectselected = {projects[i],projects[i]}
								rope.read("projects/"..projectselected[1].."/info.lua")
								for i = 1,3 do
									sliderinfo[i][2] = math.random(0,100)
								end
								sliderinfo[5] = {-255,255,sliderspeed*2}
								sliderinfo[4] = {255,0,sliderspeed*2}
							end
						end
					end
					focusoffset = nil
				-- tests if a project has already been selected
				else
					-- data switch
					if y >= 245 and y <= 265 and x >= 5 and x <= 25 then
						if projectviewmode == "details" and info.settings ~= nil then
							projectviewmode = "settings"
						else
							projectviewmode = "details"
						end
					end

					-- project reject/accept
					if y >= 376 then
						-- return
						if x <= 55 then
							projectselected[1] = "null"
							for i = 1,3 do
								sliderinfo[i][2] = math.random(0,100)
							end
							sliderinfo[5] = {255,0,sliderspeed*2}
							sliderinfo[4] = {-255,255,sliderspeed*2}
						-- play
						elseif x >= 205 and love.filesystem.exists("projects/"..projectselected[1].."/main.lua") then
							 rope.read("projects/"..projectselected[1].."/main.lua")
						end
					end
				end
			end
		end
	end
end

-- keypressed inputs
-- typed input toggles whether you are writing or using shortcuts
function love.keypressed(key,unicode)
	if key == "f1" then
		if debug then
			debug = false
		else
			debug = true
		end
	end
end

-- function that checks if a key is down every game tickyeah
shifton = null
function keydown()
	if love.keyboard.isDown("lshift") or love.keyboard.isDown("rshift") then
		shifton = true
	else
		shifton = false
	end
end

-- function that makes sure the key your pressing is an okay typedinput
function typedinputcheck(item)
	local temp = false
	local possibleinputs = {" ","1","2","3","4","5","6","7","8","9","0","q","w","e","r","t","y","u","i","o","p","a","s","d","f","g","h","j","k","l","z","x","c","v","b","n","m"}
	
	for i,v in ipairs(possibleinputs) do
		if item == v then
			temp = true
		end
	end
	
	return temp
end



--SUPERFUNCTION
--controls all of the actions from keyboard inputs from both
--keypressed and keydown
--x and y not just used for x and y positions
--but also for x and y arguments or information for ease of customisation of this table
function keyAction(action,x,y)
	if action == "debug" then
		if debug then
			debug = false
		else
			debug = true
		end
	elseif action == "nload" then
		local temp = {}
		for i,v in ipairs(lastmod[1]) do
			local temp2,_ = love.filesystem.getLastModified(v)
			table.insert(temp,temp2)
			temp2,_ = nil,nil
		end
		for i,v in ipairs(temp) do
			if v ~= lastmod[2][i] then
				ropeRead(lastmod[1][i])
			end
		end
		temp = nil
	elseif action == "fullscreen" then
		love.graphics.toggleFullscreen()
	elseif action == "escapepress" then
		if area == "menu" then
			love.event.push('quit')
		elseif area == "sandbox" then
			if sliderinfo[1].b == -220 then
				menushow = true
				currenttable = sandboxmenutable
				sliderinfo[1].b = 0
				menuwidth = 200
			elseif sliderinfo[1].b == 0 then
				menushow = false
				currenttable = sandboxtable
				sliderinfo[1].b = -220
			end
		end
	elseif action == "camera_keydir" then
		keydir = {x,y}
	elseif action == "typedinput" then
		local target = "null"
		if currenttable == synthmenutable then
			if x == "backspace" or x == "delete" then
				table.remove(typed)
				synth.info.name = table.concat(typed)
			else
				if font24:getWidth(table.concat(typed)) <= 185 and typedinputcheck(x) then
					if shifton then
						table.insert(typed,string.upper(x))
					else
						table.insert(typed,x)
					end
					synth.info.name = table.concat(typed)
				end
			end
		end
	elseif action == "ctt" then
		if x then
			local increase = 0
			if ctt[2] + 1 <= #tiletypes then
				increase = ctt[2] + 1
			elseif ctt[2] + 1 > #tiletypes then
				increase = 1
			end
			ctt = {tiletypes[increase][1],increase}
		elseif not x then
			local decrease = 0
			if ctt[2] - 1 > 0 then
				decrease = ctt[2] - 1
			elseif ctt[2] - 1 < 1 then
				decrease = #tiletypes
			end
			ctt = {tiletypes[decrease][1],decrease}
		end
		if area == "edit_map" then
			menu = {sheet = {},quad = {},scale = {}}
			for i = 1,3 do
				menu.sheet[i], menu.quad[i] = getmenudata(i-2)
				local foo, foo, foo, scale = menu.quad[i]:getViewport()
				menu.scale[i] = 100 / scale
			end
		end
	elseif action == "ctp" then
		local test = null
		if ctt[1] == "tile" then
			test = #spritesheets.quadInfo
		elseif ctt[1] == "overlay" then
			test = #spritesheets.overlayquadInfo
		elseif ctt[1] == "object" then
			test = #spritesheets.objectquadInfo
		end
		assert(test ~= null,"TEST == NULL (control.lua(548))")
		if x then
			local increase = 0
			if ctp[ctt[2]] + 1 <= test then
				increase = ctp[ctt[2]] + 1
			elseif ctp[ctt[2]] + 1 > test then
				increase = 1
			end
			ctp[ctt[2]] = increase
		elseif not x then
			local decrease = 0
			if ctp[2] - 1 > 0 then
				decrease = ctp[ctt[2]] - 1
			elseif ctp[2] - 1 < 1 then
				decrease = test
			end
			ctp[ctt[2]] = decrease
		end
		
		menu = {sheet = {},quad = {},scale = {}}
		for i = 1,3 do
			menu.sheet[i], menu.quad[i] = getmenudata(i-2)
			local foo, foo, foo, scale = menu.quad[i]:getViewport()
			menu.scale[i] = 100 / scale
		end
		
	elseif action == "tilesize" then
		tilesize = tilesize + x
	elseif action == "backgroundshade" then
		if backgroundshade >= 255 then
			backgroundshade = 254
		elseif backgroundshade <= 0 then
			backgroundshade = 1
		elseif backgroundshade <= 255 and backgroundshade >= 0 then
			backgroundshade = backgroundshade + x
		end
	elseif action == "map_save" then
		mapsave()
	end
end










