--[[
LICENCE DETAILS: ZLIB LICENCE

Copyright (c) <2011> <Lewis Pearce>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution.
]]

---- EXAMPLE OF HOW TO USE THIS TABLE
----
---- require "ow_rope.lua"
---- libraries = {"illusion.lua"}
---- ropeLoad("exampledirectory","examplesavefile")
----

rope = {}

function rope.load(directory,savefile)
	math.randomseed(os.time())	
	
	-- you can put all the libraries your game loads here
	
	love.filesystem.setIdentity(directory)
	-- >>>
	if love.filesystem.exists(savefile) then 
		rope.read(savefile)
	elseif not love.filesystem.exists(savefile) then
	-- place what you want the default settings for this file here
		love.filesystem.mkdir("sandbox")
		love.filesystem.mkdir("challenge")
	
		settings = {
			fullscreen=false,
			premenu = true,
			colour = {{255,255,255},{0,0,0}},
			currentres = {800,600,1},
			canvases = true,
			volume = 0.3,
			subsettings = {
			showbranchpos = false
			}
		}
		--use this bit to save in other ways
		love.filesystem.write(savefile,"settings="..rope.burn(settings))
	end
	
	-- <<<
	-- if you need more files copy and paste the stuff between >>> and <<<
	-- modify savefile and directory so they become savefile1 and directory1
	-- then add them to the box next to "ropeLoad" and where you call it in the love.load
end

-- tables must be compatible with rope.burn

function rope.burn(t)
	local data = {}
	if type(t) == "number" then
		return t
	elseif type(t) == "boolean" then
		if t then
			return "true"
		elseif not t then
			return "false"
		end
	elseif type(t) == "string" then
		return t
	elseif type(t) == "table" then
		local data = {}
		table.insert(data,"{\n")
		for k,v in pairs(t) do
			if type(k) ~= "number" and type(v) ~= "function" then
				local k_info = rope.burn(k)
				local v_info = rope.burn(v)
				if type(v) == "string" then
					v_info = '"'..v_info..'"'
				end
				table.insert(data,k_info)
				table.insert(data,"=")
				table.insert(data,v_info)
				table.insert(data,",\n")
				k_info,v_info = nil,nil
			elseif type(k) == "number" and type(v) == "table" then
				local v_info = rope.burn(v)
				table.insert(data,v_info)
				table.insert(data,",\n")
				v_info = nil
			elseif type(k) == "number" and type(v) == "number" then
				local v_info = rope.burn(v)
				table.insert(data,v_info)
				table.insert(data,",\n")
				v_info = nil
			elseif type(k) == "number" and type(v) == "boolean" then
				local v_info = rope.burn(v)
				table.insert(data,v_info)
				table.insert(data,",\n")
				v_info = nil
			elseif type(k) == "number" and type(v) == "string" then
				local v_info = rope.burn(v)
				table.insert(data,'"'..v_info..'"')
				table.insert(data,",\n")
				v_info = nil
			end
		end
		table.insert(data,"\n}")
		return table.concat(data)
	end
end

function rope.read(item)
	love.filesystem.load(item)()
end

function rope.readandreturn(item)
	return love.filesystem.load(item)()
end