-- This function will return a string filetree of all files
-- in the folder and files in all subfolders
function recursiveEnumerate(folder, fileTree)
    local lfs = love.filesystem
    local filesTable = lfs.enumerate(folder)
    for i,v in ipairs(filesTable) do
        local file = folder.."/"..v
        if lfs.isFile(file) then
            fileTree = fileTree.."\n"..file
        elseif lfs.isDirectory(file) then
            fileTree = fileTree.."\n"..file.." (DIR)"
            fileTree = recursiveEnumerate(file, fileTree)
        end
    end
    return filesTable
end

function closest(x,y,ltable,pos1,pos2)
	local pos1 = pos1 or 1
	local pos2 = pos2 or 2
	local closest = {1,100000}
	local testx = 0
	local testy = 0
	for i = 1,#ltable do
		testx = math.abs(ltable[i][pos1] - x)
		testy = math.abs(ltable[i][pos2] - y)
		testz = math.abs(testx + testy)
		if testz < closest[2] then
			closest = {i,testz}
		end
	end
	return closest[1]
end

function closestleaf(x,y,ltable,pos1,pos2)
	local pos1 = pos1 or 1
	local pos2 = pos2 or 2
	local closest = {1,100000}
	local testx = 0
	local testy = 0
	local temp = false
	for i = 1,#ltable do
		for j = 1,#ltable[i].leaves do
			if #ltable[i].leaves ~= 0 then
				testx = math.abs(ltable[i].leaves[j][pos1] - x)
				testy = math.abs(ltable[i].leaves[j][pos2] - y)
				testz = math.abs(testx + testy)
				if testz < closest[2] then
					closest = {i,testz,j}
				end
				temp = true
			end
		end
	end
	if temp then
		return closest[1],closest[3]
	else
		return 0,0
	end
end

function variableupdate(dt)

	mX = love.mouse.getX()
	mY = love.mouse.getY()
	mXt, foo = math.modf((mX/Z[1])+camera.x)
	mYt, foo = math.modf((mY/Z[1])+camera.y)
	
	-- TOOLTIPS INTERFACE CODE
	-- large sections use the hard coded version short bottom section the inbuilt faster one
	-- larger version because of limitations of the shorter.
	if currenttable[nearestMenu(mX,mY)].tip == "variablecanvasarea" then
		if synth.area == "menu" then
			if mX >= 20 and mX <= 130 and mY >= 150 and mY <= 175 then
				tooltip = "Create a new Sample using the built in synthesizer."
			elseif mX >= 20 and mX <= 130 and mY >= 200 and mY <= 220 then
				tooltip = "Import a soundfile from the samples folder."
			else
				tooltip = null
			end
		elseif synth.area == "s_stage1" then
			if mX >= 7 and mX <= 37 and mY >= 110+synth.canvasy and mY <= 140+synth.canvasy then
				tooltip = "Squarewave Oscillator."
			elseif mX >= 42 and mX <= 72 and mY >= 110+synth.canvasy and mY <= 140+synth.canvasy then
				tooltip = "Triangle Oscillator."
			elseif mX >= 77 and mX <= 107 and mY >= 110+synth.canvasy and mY <= 140+synth.canvasy then
				tooltip = "Sawtooth Oscillator."
			elseif mX >= 112 and mX <= 142 and mY >= 110+synth.canvasy and mY <= 140+synth.canvasy then
				tooltip = "Sinewave Oscillator."
			elseif mX >= 147 and mX <= 177 and mY >= 110+synth.canvasy and mY <= 140+synth.canvasy then
				tooltip = "White noise Oscillator."
			elseif mX >= 182 and mX <= 212 and mY >= 110+synth.canvasy and mY <= 140+synth.canvasy then
				tooltip = "Pink noise Oscillator."
			elseif mX >= 217 and mX <= 267 and mY >= 110+synth.canvasy and mY <= 140+synth.canvasy then
				tooltip = "Oscillators, the starting shape of the soundwave."
			elseif mX >= 5 and mX <= 115 and mY >= 145+synth.canvasy and mY <= 165+synth.canvasy then
				tooltip = "Note and pitch control, select the note on the keyboard and raise or lower the octave of the note with the arrows either side of the current octave number."
			elseif mX >= 200 and mX <= 215 and mY >= 145+synth.canvasy and mY <= 165+synth.canvasy then
				tooltip = "Lower the octave."
			elseif mX >= 235 and mX <= 250 and mY >= 145+synth.canvasy and mY <= 165+synth.canvasy then
				tooltip = "Raise the octave."
			elseif mX >= 7 and mX <= 37 and mY >= 250+synth.canvasy and mY <= 280+synth.canvasy then
				tooltip = "8 bit sound."
			elseif mX >= 42 and mX <= 72 and mY >= 250+synth.canvasy and mY <= 280+synth.canvasy then
				tooltip = "16 bit sound."
			elseif mX >= 77 and mX <= 107 and mY >= 250+synth.canvasy and mY <= 280+synth.canvasy then
				tooltip = "Control for the number of Bits."
			elseif mX >= 180 and mX <= 215 and mY >= 250+synth.canvasy and mY <= 280+synth.canvasy then
				tooltip = "Shorten the length."
			elseif mX >= 230 and mX <= 265 and mY >= 250+synth.canvasy and mY <= 280+synth.canvasy then
				tooltip = "Lengthen."
			elseif mX >= 250 and mX <= 315 and mY >= 250+synth.canvasy and mY <= 280+synth.canvasy then
				tooltip = "Alter the length of the sample created with the 4 arrows."
			elseif mX >= 295 and mX <= 325 and mY >= 290+synth.canvasy+(#synth.info.effects*100) and mY <= 320+synth.canvasy+(#synth.info.effects*100) then
				local temp = true
				for i = 1,#synth.info.effects do
					if synth.info.effects[i].effect == "blank" then
						temp = false
					end
				end
				if temp then
					tooltip = "Add an effect."
				else
					tooltip = "You already have a new effect, finish setting that up before you add another."
				end
				temp = nil
			else
				tooltip = null
			end
		end
	elseif currenttable[nearestMenu(mX,mY)].tip ~= null then
		tooltip = currenttable[nearestMenu(mX,mY)].tip
	else
		tooltip = null
	end
	
	-- closest part calculations
	if currenttable == sandboxtable then
		if #branches == 0 then
			cle = 0
		else
			cle = closest(mX,mY,branches,3,4)
		end
	else
		cle = 0
	end
	if tool ~= "branch" then
		if #branches == 0 then
			cl = {0,0}
		elseif #branches ~= 0 then
			cl = {closestleaf(mX,mY,branchinfo)}
		else
			cl = {0,0}
		end
	else
		cl = {0,0}
	end
	
	if area == "pre-menu" then
		if love.mouse.isDown("l") then		
			if mX > 40 and mX < 295 and mY > 210 and mY < 341 then
				mouseAction(mX,mY,1)
			end
		end
	elseif currenttable == synthmenutable then
		if synth.area == "s_stage1" or synth.area == "a_stage1" then
			if love.mouse.isDown("l") then
				if mX > 379 and mX < 391 and mY > 89 and mY < 111 then
					mouseAction(mX,mY,1)
				elseif mX > 379 and mX < 391 and mY > 429 and mY < 449 then
					mouseAction(mX,mY,1)
				end
			end
		end
	elseif currenttable == sandboxtable then
		if love.mouse.isDown("l") then
			if mX > 96 and mX < 108 and mY > (screenHeight-26) then
				mouseAction(mX,mY,1)
			elseif mX > (screenWidth-14) and mY > (screenHeight-26) then
				mouseAction(mX,mY,1)
			end
		end
	end
	
	--[[
	if area == "pre-menu" then
		if currenttable[nearestMenu(mX,mY)].tip ~= null then
			tooltip = currenttable[nearestMenu(mX,mY)].tip
		else
			tooltip = null
		end
	else
		if nearestMenu(mX,mY,windows) == null then
			if currenttable[nearestMenu(mX,mY)].tip ~= null then
				tooltip = currenttable[nearestMenu(mX,mY)].tip
			else
				tooltip = null
			end
		else
			tooltip = null
		end
	end
	]]
end

function awolUpdate(area)
	if area == "pre-menu" then
		awol.new("Developed by",10,20,780,120,{rgb={75,80,195},settings = {minimisable = false, removable = false, movable = false, resizable = false,}})
		awol.new("Settings",10,160,280,430,{rgb={75,80,195},settings = {minimisable = false, removable = false, movable = false, resizable = false,}})
		awol.new("Module",510,160,280,430,{rgb={75,80,195},settings = {minimisable = false, removable = false, movable = false, resizable = false,}})
		awol.new("Pre-load",310,310,180,280,{rgb={75,80,195},settings = {minimisable = false, removable = false, movable = false, resizable = false,}})
	end
end

function descendancy(tpos) --tpos = table position
	local output = 0
	
	for i,v in ipairs(branches) do
		if v[1] == branches[tpos][3] and v[2] == branches[tpos][4] then
			output = output + 1
			output = output + descendancy(i)
		end
	end
	
	return output
end

function heightset (tpos)
	for i,v in ipairs(branches) do
		if v[1] == branches[tpos][3] and v[2] == branches[tpos][4] then
			branchinfo[i].pos = branchinfo[tpos].pos + 1
			heightset(i)
		end
	end
end

function getmaxheight()
	local temp = 0
	for i,v in ipairs(branchinfo) do
		if v.pos > temp then
			temp = v.pos
		end
	end
	return temp
end

function returnoscillator(data)
	if data == 1 then
		return Moan.osc.square
	elseif data == 2 then
		return Moan.osc.triangle
	elseif data == 3 then
		return Moan.osc.saw
	elseif data == 4 then
		return Moan.osc.sin
	elseif data == 5 then
		return Moan.osc.wn
	elseif data == 6 then
		return Moan.osc.pn
	end
end

function synth.rendersound(data)
	return love.audio.newSource(Moan.newSample(Moan.envelope(returnoscillator(data.osc)( Moan.pitch(data.note, data.octave) )), data.len, data.samplerate, data.bits))
end

-- manages loading and playing all the sounds on the tree
local bpmpos = 0
local tempaudiopos = {}
local tempaudioinfo = {}
local repeatpos = 0
function synth.audiomanager(dt)
	-- if playing
	if playingaudio then
		-- if pos to play
		if bpmpos > 60 then
			bpmpos = 0
			sounds.info.activebranches = tempaudiopos
			tempaudiopos = {}
			for i,v in ipairs(tempaudioinfo) do
				if v[1] ~= 0 then
					love.audio.stop(sounds.data[v[1]].sound)
					sounds.data[v[1]].sound:setVolume(settings.volume+v[3])
					sounds.data[v[1]].sound:setPitch(v[2])
					love.audio.play(sounds.data[v[1]].sound)
				end
			end
			tempaudioinfo = {}
			sounds.info.pos = sounds.info.pos + 1
			if sounds.info.pos > repeatpos then
				sounds.info.pos = 1
			end
		-- if pos to load next play
		elseif bpmpos > sounds.info.bpm/60-(sounds.info.bpm/100) then
			tempaudiopos = findvar(branchinfo,pos,sounds.info.pos)
			for i,v in ipairs(tempaudiopos) do
				local temp = {}
				for i,v in ipairs(branchinfo[v].leaves) do
					table.insert(temp,{v[3],v[4],v[5]})
				end
				for j,w in ipairs(temp) do
					if not isintable(w[1],tempaudioinfo,1) then
						table.insert(tempaudioinfo,w)
					end
				end
				temp = nil
			end
			repeatpos = getmaxheight()
		end
		bpmpos = bpmpos + (dt*sounds.info.bpm)
	end
end

function templo()
	return #tempaudiopos,#tempaudioinfo
end
