--[[
LICENCE DETAILS: ZLIB LICENCE

Copyright (c) <2011> <Lewis Pearce>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution.
]]

--[[ 
----------------------------
THE AWOL LIBRARY VERSION 004
----------------------------

This library is a short as possible attempt to write a simple library that allows easily
modifiable windows with full interactions with the minimum functions required within your program.

It can be used for popup windows, option screens or any application that needs windows 
and i have designed it to use as much hard code areas as possible because although this removes some
usability for early coders it makes it more usefull for me and hopefully anyone else who uses it.

-------------------------------------
Instructions on how to use it follow:
-------------------------------------

1: First you must add these to your appropriate areas
-----------------------------------------------------

	local awol = require "ow_awol.lua"
		to above or within love.load most likely
	awol.input(dt)
		to love.update(dt)
	awol.draw()
		to love.draw()

2: Now define windows to run, this can be done in 2 ways.
---------------------------------------------------------
	
	by adding a awol.new(...) function call wherever
	this allows new windows to be created on the fly.
	example: awol.new("Example",200,200,200,200,{rgb={200,200,200},settings={minimisable = true, removable = true, movable = true, resizable = true,},startminimised = false,framebufferwindow = false})
=	(last 2 in info if false can be left off)
	
	or by adding the windows to the windows table directly
	this way they will open and load on startup.
	example:
	{
		z = 0,
		coords = {200,200,300,170},
		theme = {0,200,0},
		header = "EXAMPLE INFO",
		settings = {
			minimisable = true,
			removable = true,
			movable = true,
			resizable = true,
		},
		minimised = false,
		framebuffer = love.graphics.newFrameBuffer(coords[3],coords[4]), --THIS ONE ONLY NEEDED IF FRAMEBUFFER WINDOW
		resize = {coords[3],coords[4]},  --SAME AS ABOVE
	},

3: Add draw content to awolContent(...)
---------------------------------------
	
	heres an example of a normal input
	and ive added how to use scissors or framebuffers below
	
	NORMAL:
	-------
	
	if title == "EXAMPLE INFO" then
		
		love.graphics.line(xywh[1],xywh[2],xywh[1] + xywh[3],xywh[2] + xywh[4])
		love.graphics.line(xywh[1],xywh[2],xywh[1] + xywh[3]-10,xywh[2] + xywh[4])
		love.graphics.line(xywh[1],xywh[2],xywh[1] + xywh[3]-20,xywh[2] + xywh[4])
		
		love.graphics.line(xywh[1] + xywh[3],xywh[2],xywh[1],xywh[2] + xywh[4])
		love.graphics.line(xywh[1] + xywh[3],xywh[2],xywh[1]+10,xywh[2] + xywh[4])
		love.graphics.line(xywh[1] + xywh[3],xywh[2],xywh[1]+20,xywh[2] + xywh[4])
	end
	
	FRAMEBUFFERS: (ALTHOUGH IN ALMOST ALL CASES USE SCISSORS THEY ARE FASTER AND BETTER)
	-------------
	
	if title == "EXAMPLE INFO" then
		love.graphics.setRenderTarget(windows[I].framebuffer)
		
		love.graphics.line(xywh[1],xywh[2],xywh[1] + xywh[3],xywh[2] + xywh[4])
		love.graphics.line(xywh[1],xywh[2],xywh[1] + xywh[3]-10,xywh[2] + xywh[4])
		love.graphics.line(xywh[1],xywh[2],xywh[1] + xywh[3]-20,xywh[2] + xywh[4])
		
		love.graphics.line(xywh[1] + xywh[3],xywh[2],xywh[1],xywh[2] + xywh[4])
		love.graphics.line(xywh[1] + xywh[3],xywh[2],xywh[1]+10,xywh[2] + xywh[4])
		love.graphics.line(xywh[1] + xywh[3],xywh[2],xywh[1]+20,xywh[2] + xywh[4])
		
		love.graphics.setRenderTarget()
		love.graphics.draw(windows[I].framebuffer,xywh[1],xywh[2])
	end
	
	SCISSORS:
	---------
	
	if title == "EXAMPLE INFO" then
		love.graphics.setScissor(xywh[1],xywh[2],xywh[3],xywh[4])
	
		love.graphics.line(xywh[1],xywh[2],xywh[1] + xywh[3],xywh[2] + xywh[4])
		love.graphics.line(xywh[1],xywh[2],xywh[1] + xywh[3]-10,xywh[2] + xywh[4])
		love.graphics.line(xywh[1],xywh[2],xywh[1] + xywh[3]-20,xywh[2] + xywh[4])
		
		love.graphics.line(xywh[1] + xywh[3],xywh[2],xywh[1],xywh[2] + xywh[4])
		love.graphics.line(xywh[1] + xywh[3],xywh[2],xywh[1]+10,xywh[2] + xywh[4])
		love.graphics.line(xywh[1] + xywh[3],xywh[2],xywh[1]+20,xywh[2] + xywh[4])
	
		love.graphics.setScissor()
	end
	
	yep its as simple as that just use the normal draw commands
	all the xywh's are as follows
	1 == left x side of the window
	2 == top y side of the window
	3 == width
	4 == height
	
-------------------------
--------FINAL NOTE-------
-------------------------

Looks best with a font sized about 14 :)

The code within love.mousepressed may need to be tucked into your programs love.mousepressed so it doesnt get overwritten.

Although i am aware this may not be the slickest library ever i hope its usefull to some people
and it should be relatively easy to modify everything about it.

]]

windows = {}

local function draw()
	for i = 1,#windows do
		awolWindow(windows[i],i)
	end
end


-- function for creating windows table entries
-- submit all as tables except headertext, startminimised and framebufferwindow
-- they should be submitted as true/false arguments
-- example
-- awol.new("EXAMPLEDEFAULT",200,200,200,200)
-- awol.new("Example",200,200,200,200,{rgb={200,200,200},settings={minimisable = true, removable = true, movable = true, resizable = true,},startminimised = false,framebufferwindow = false})

local function new(title,x,y,w,h,info)
	framebufferwindow = info.framebufferwindow or false

	local output = {}
	output.z = 0
	output.coords = {x,y,w,h}
	output.theme = info.rgb or {75,80,195}
	output.header = title
	output.settings = info.settings or {minimisable = true, removable = true, movable = true, resizable = true,}
	output.minimised = info.startminimised or false
	if framebufferwindow then
		output.framebuffer = love.graphics.newFramebuffer(w,h)
		output.resize = {w,h}
	end
	table.insert(windows,output)
	output = nil
end

-- basic drawing code for the windows
-- change the polygon data for different shaped tops
-- but the code is only checking for square areas of 10 by width so you cant go to crazy unless you did deeper and modify it lots
function awolWindow(table,I)
	local x = table.coords[1]
	local y = table.coords[2]
	local width = table.coords[3]
	local height = table.coords[4]
	local theme = table.theme
	local header = table.header
	local settings = table.settings
	local minimised = table.minimised
	
	if not minimised then
		love.graphics.setColor(20,20,20)
		love.graphics.rectangle("fill",x,y,width,height)
		awolContent(header,{x,y,width,height},I)
	end
	love.graphics.setColor(theme[1],theme[2],theme[3])
	love.graphics.polygon("fill",x+5,y-10,x+width-5,y-10,x+width,y-5,x+width,y,x,y,x,y-5)
	love.graphics.setColor(theme[1]+10,theme[2]+10,theme[3]+10)
	love.graphics.polygon("line",x+5,y-10,x+width-5,y-10,x+width,y-5,x+width,y,x,y,x,y-5)
	if not minimised then
		if table.resize == null then
			love.graphics.rectangle("line",x,y,width,height)
		else
			love.graphics.rectangle("line",x,y,table.resize[1],table.resize[2])
		end
	end
	
	love.graphics.setColor(255,255,255)
	love.graphics.print(header,x+5,y-10)
	if settings.minimisable then
		love.graphics.print("~",x+width-25,y-10)
	end
	if settings.removable then
		love.graphics.print("X",x+width-10,y-10)
	end
	if settings.resizable and not minimised then
		love.graphics.setColor(theme[1]+10,theme[2]+10,theme[3]+10)
		love.graphics.polygon("fill",x+width,y+height-10,x+width,y+height,x+width-10,y+height)
		love.graphics.setColor(255,255,255)
		love.graphics.print(".",x+width-4,y+height-14)
	end
	love.graphics.setColor(255,255,255)
end

-- gets the title of the table and draws the content from that name
-- contain draw information in functions from here or within this table itself
function awolContent(title,xywh,I)
	love.graphics.setColor(255,255,255)
	if title == "Developed by" then
		love.graphics.setFont(font24)
		
		love.graphics.setColor(65,70,185)
		love.graphics.rectangle("fill",xywh[1],xywh[2]+5,xywh[3],8)
		love.graphics.rectangle("fill",xywh[1],xywh[2]+xywh[4]-13,xywh[3],8)
		love.graphics.setColor(45,50,165)
		love.graphics.rectangle("fill",xywh[1],xywh[2]+18,xywh[3],6)
		love.graphics.rectangle("fill",xywh[1],xywh[2]+xywh[4]-24,xywh[3],6)
		love.graphics.setColor(25,30,145)
		love.graphics.rectangle("fill",xywh[1],xywh[2]+29,xywh[3],4)
		love.graphics.rectangle("fill",xywh[1],xywh[2]+xywh[4]-33,xywh[3],4)
		love.graphics.setColor(5,10,125)
		love.graphics.rectangle("fill",xywh[1],xywh[2]+38,xywh[3],2)
		love.graphics.rectangle("fill",xywh[1],xywh[2]+xywh[4]-40,xywh[3],2)
		love.graphics.setColor(255,255,255)
		
		love.graphics.draw(obscurelogo,xywh[1]+10,xywh[2]+10,0,3)
		love.graphics.print("For",xywh[1]+380,xywh[2]+45)
		love.graphics.draw(lovelogo,xywh[1]+520,xywh[2]-10)
		
		love.graphics.setFont(font)
	elseif title == "Settings" then
		love.graphics.setFont(font24)
		
		love.graphics.setColor(75,80,195)
		love.graphics.polygon("fill",xywh[1]+10,xywh[2]+10,xywh[1]+xywh[3]-20,xywh[2]+10,xywh[1]+xywh[3]-10,xywh[2]+20,xywh[1]+xywh[3]-10,xywh[2]+70,xywh[1]+20,xywh[2]+70,xywh[1]+10,xywh[2]+60)
		love.graphics.polygon("fill",xywh[1]+10,xywh[2]+80,xywh[1]+xywh[3]-20,xywh[2]+80,xywh[1]+xywh[3]-10,xywh[2]+90,xywh[1]+xywh[3]-10,xywh[2]+140,xywh[1]+20,xywh[2]+140,xywh[1]+10,xywh[2]+130)
		love.graphics.setColor(255,255,255)
		
		love.graphics.print("Screen Resolution:",xywh[1]+10,xywh[2]+10)
		love.graphics.print(options.resolution[1].." x "..options.resolution[2],xywh[1]+40,xywh[2]+34)
		love.graphics.rectangle("line",xywh[1]+xywh[3]-70,xywh[2]+40,20,20)
		love.graphics.polygon("line",xywh[1]+xywh[3]-65,xywh[2]+50,xywh[1]+xywh[3]-55,xywh[2]+45,xywh[1]+xywh[3]-55,xywh[2]+55)
		love.graphics.rectangle("line",xywh[1]+xywh[3]-40,xywh[2]+40,20,20)
		love.graphics.polygon("line",xywh[1]+xywh[3]-35,xywh[2]+45,xywh[1]+xywh[3]-25,xywh[2]+50,xywh[1]+xywh[3]-35,xywh[2]+55)
		
		love.graphics.print("FullScreen:",xywh[1]+10,xywh[2]+80)
		love.graphics.rectangle("line",xywh[1]+xywh[3]-40,xywh[2]+110,20,20)
		if options.fullscreen then
			love.graphics.print("X",xywh[1]+xywh[3]-35,xywh[2]+107)
		end
		
		love.graphics.setFont(font)
	elseif title == "Module" then
		love.graphics.setFont(font24)
		
		love.graphics.setColor(75,80,195)
		for i = 1,#modules do
			love.graphics.polygon("fill",xywh[1]+10,xywh[2]+10+(70*(i-1)),xywh[1]+xywh[3]-20,xywh[2]+10+(70*(i-1)),xywh[1]+xywh[3]-10,xywh[2]+20+(70*(i-1)),xywh[1]+xywh[3]-10,xywh[2]+70+(70*(i-1)),xywh[1]+20,xywh[2]+70+(70*(i-1)),xywh[1]+10,xywh[2]+60+(70*(i-1)))
		end
		love.graphics.setColor(255,255,255)
		
		for i = 1,#modules do
			love.graphics.print(modules[i],xywh[1]+10,xywh[2]+10+(70*(i-1)))
			love.graphics.rectangle("line",xywh[1]+xywh[3]-40,xywh[2]+40+(70*(i-1)),20,20)
		end
		
		love.graphics.print("X",xywh[1]+xywh[3]-35,xywh[2]+37+(70*(options.module-1)))
		
		love.graphics.setFont(font)
	elseif title == "Pre-load" then
		love.graphics.setFont(font24)
		love.graphics.setColor(75,80,195)
		love.graphics.polygon("fill",xywh[1]+10,xywh[2]+10,xywh[1]+xywh[3]-20,xywh[2]+10,xywh[1]+xywh[3]-10,xywh[2]+20,xywh[1]+xywh[3]-10,xywh[2]+70,xywh[1]+20,xywh[2]+70,xywh[1]+10,xywh[2]+60)
		love.graphics.setColor(255,255,255)
		
		love.graphics.print("Cartographer",xywh[1]+25,xywh[2]+15)
		love.graphics.print("LAUNCH",xywh[1]+55,xywh[2]+39)
		
		love.graphics.setFont(font)
	end
end

-- This lower code just deals with the mouse detection on the window and the window edges
awolI = {
	window = 0,
	pos = {},
	offset = {},
}

--[[
function love.mousepressed(x,y,button)
	if button == "l" then
		if awolMouseCollider(x,y,2) ~= null then
			for i = 1,#windows do
				windows[i].z = 0
			end
			windows[awolMouseCollider(x,y,2)].z = 1
			table.sort(windows,awolSort)
		end
		if awolMouseCollider(x,y) ~= null then
			awolI.window = awolMouseCollider(x,y)
			awolI.pos = {windows[awolI.window].coords[1],windows[awolI.window].coords[2]}
			awolI.offset = {x-awolI.pos[1],y-awolI.pos[2]}
			if awolCollision(x,y,1,1,(awolI.pos[1]+windows[awolI.window].coords[3])-25,awolI.pos[2]-10,25,10) then
				if awolCollision(x,y,1,1,(awolI.pos[1]+windows[awolI.window].coords[3])-25,awolI.pos[2]-10,10,10) then
					if not windows[awolI.window].minimised then
						windows[awolI.window].minimised = true
					else
						windows[awolI.window].minimised = false
					end
				elseif awolCollision(x,y,1,1,(awolI.pos[1]+windows[awolI.window].coords[3])-10,awolI.pos[2]-10,10,10) then
					table.remove(windows,awolI.window)
				end
				awolI = {
					window = 0,
					pos = {},
					offset = {},
				}
			end
		elseif awolMouseCollider(x,y,1) ~= null then
			awolI.window = {awolMouseCollider(x,y,1)}
			awolI.pos = {windows[awolI.window[1] ].coords[3],windows[awolI.window[1] ].coords[4]}
			awolI.offset = {x-awolI.pos[1],y-awolI.pos[2]}
		end
	end
end
]]

function awolMouseCollider(x,y,mode)
	local vx,vy,vw,vh
	local current = null
	for i,v in ipairs(windows) do
		if mode == 1 then
			vx = v.coords[1]+v.coords[3]-10
			vy = v.coords[2]+v.coords[4]-10
			vw = 10
			vh = 10
		elseif mode == 2 then
			vx = v.coords[1]
			vy = v.coords[2]-10
			vw = v.coords[3]
			vh = v.coords[4]+10
		elseif mode == null or v.minimised == true then
			vx = v.coords[1]
			vy = v.coords[2]-10
			vw = v.coords[3]
			vh = 10
		end
		
		if awolCollision(x,y,1,1,vx,vy,vw,vh) == true then
			current = i
		end
	end
	return current
end

local function input(dt)
	local x,y = love.mouse.getX(),love.mouse.getY()
	local awolMC_result = awolMouseCollider(x,y)
	
	if love.mouse.isDown("l") then
		if awolI.window ~= 0 then
			if type(awolI.window) == "number" then
				if windows[awolI.window].settings.movable then
					windows[awolI.window].coords[1] = x - awolI.offset[1]
					windows[awolI.window].coords[2] = y - awolI.offset[2]
					
					if windows[awolI.window].coords[1] < 0 then
						windows[awolI.window].coords[1] = 0
					elseif windows[awolI.window].coords[1]+windows[awolI.window].coords[3] > screenWidth then
						windows[awolI.window].coords[1] = screenWidth - windows[awolI.window].coords[3]
					end
					
					if windows[awolI.window].coords[2]-10 < 10 then
						windows[awolI.window].coords[2] = 10
					elseif windows[awolI.window].coords[2]+windows[awolI.window].coords[4] > screenHeight then
						windows[awolI.window].coords[2] = screenHeight - windows[awolI.window].coords[4]
					end
				end
			elseif type(awolI.window) == "table" then
				if windows[awolI.window[1]].settings.resizable then
					if windows[awolI.window[1]].resize == null then
						windows[awolI.window[1]].coords[3] = x - awolI.offset[1]
						windows[awolI.window[1]].coords[4] = y - awolI.offset[2]
						
						if windows[awolI.window[1]].coords[3] < font:getWidth(windows[awolI.window[1]].header) + 35 then
							windows[awolI.window[1]].coords[3] = font:getWidth(windows[awolI.window[1]].header) + 35
						end
						
						if windows[awolI.window[1]].coords[4] < 10 then
							windows[awolI.window[1]].coords[4] = 10
						end
						
					else
						windows[awolI.window[1]].resize[1] = x - awolI.offset[1]
						windows[awolI.window[1]].resize[2] = y - awolI.offset[2]
						
						if windows[awolI.window[1]].resize[1] < font:getWidth(windows[awolI.window[1]].header) + 35 then
							windows[awolI.window[1]].resize[1] = font:getWidth(windows[awolI.window[1]].header) + 35
						end
						
						if windows[awolI.window[1]].resize[2] < 10 then
							windows[awolI.window[1]].resize[2] = 10
						end
					end
				end
			end
		end
	elseif not love.mouse.isDown("l") then
		if awolI.window ~= 0 then
			if type(awolI.window) == "table" then
				if windows[awolI.window[1]].framebuffer ~= null then
					windows[awolI.window[1]].coords[3] = windows[awolI.window[1]].resize[1]
					windows[awolI.window[1]].coords[4] = windows[awolI.window[1]].resize[2]
					windows[awolI.window[1]].framebuffer = love.graphics.newFramebuffer(windows[awolI.window[1]].coords[3],windows[awolI.window[1]].coords[4])
				end
			end
			awolI = {
				window = 0,
				pos = {},
				offset = {},
			}
		end
	end
end

function awolCollision(box1x, box1y, box1w, box1h, box2x, box2y, box2w, box2h)
    if box1x > box2x + box2w - 1 or -- Is box1 on the right side of box2?
       box1y > box2y + box2h - 1 or -- Is box1 under box2?
       box2x > box1x + box1w - 1 or -- Is box2 on the right side of box1?
       box2y > box1y + box1h - 1    -- Is b2 under b1?
    then
        return false                -- No collision. Yay!
    else
        return true                 -- Yes collision. Ouch!
    end
end

function awolSort (a,b)
	return a.z<b.z
end

--MODULE TABLE
return {
	new = new,
	draw = draw,
	input = input,
}
-- END MODULE TABLE