--[[
LICENCE DETAILS: ZLIB LICENCE

Copyright (c) <2011> <Lewis Pearce>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution.
]]

sip = {
version = 1,
info = {power = false},
data = {input = {},output = {}},
}

sipfont = love.graphics.newFont(11)

function sip.power()
	if sip.info.power then
		sip.info.power = false
	else
		sip.info.power = true
	end
end

function sip.display()
	if sip.info.power then
		local r, g, b, a = love.graphics.getColor()
		local font = love.graphics.getFont()
		love.graphics.setColor(0,0,0,200)
		love.graphics.setFont(sipfont)
		love.graphics.rectangle("fill",0,0,love.graphics.getWidth(),love.graphics.getHeight())
		love.graphics.setColor(255,255,255,255)
		love.graphics.print("--[[ S.I.P, Synergetic Interface Program, Version "..sip.version..". Type help for info. ]]",0,0)
		love.graphics.print(table.concat(sip.data.input),5,11)
		if #sip.data.input == 0 then
			local temp = 1
			for i = #sip.data.output,1,-1 do
				love.graphics.print(sip.data.output[i],5,11*temp)
				temp = temp + 1
			end
		else
			local temp = 2
			for i = #sip.data.output,1,-1 do
				love.graphics.print(sip.data.output[i],5,11*temp)
				temp = temp + 1
			end
		end
		love.graphics.setColor(r,g,b,a)
		love.graphics.setFont(font)
	end
end

-- function that makes sure the key your pressing is an okay typedinput
function sip.viableinput(item)
	local temp = false
	local possibleinputs = {"[","]","."," ","1","2","3","4","5","6","7","8","9","0","q","w","e","r","t","y","u","i","o","p","a","s","d","f","g","h","j","k","l","z","x","c","v","b","n","m"}
	
	for i,v in ipairs(possibleinputs) do
		if item == v then
			temp = true
		end
	end
	for i,v in ipairs(possibleinputs) do
		if item == string.upper(v) then
			temp = true
		end
	end
	
	return temp
end

function sip.input(key)
	local data = {}
	if key == "return" then
		if #sip.data.input ~= 0 then
			table.insert(sip.data.output,table.concat(sip.data.input))
			sip.cognition(table.concat(sip.data.input))
			sip.data.input = {}
		end
	elseif key == "delete" or key == "backspace" then
		table.remove(sip.data.input)
	else
		if sip.viableinput(key) then
			table.insert(sip.data.input,key)
		end
	end
end

function sip.stringsplit(input,iterator)
	iterator = iterator or "."
	local output = {}
	for word in input:gmatch(iterator) do 
		table.insert(output, word)
	end
	return output
end

function sip.cognition(item)
	local data = sip.stringsplit(item,"%S+")
	for i,v in ipairs(data) do
		if string.find(v,"[%.$]") then
		end
	end
	local output = {}
	if data[1] == "help" and #data == 1 then
		local temp = {
			"Input instructions:",
			"get <item>, this will return the value of global variables within your program i.e get foo;",
			"set <item> <value>, this will set the value of global variables within your program i.e set foo foo;",
			"print <item>, literally just prints everything you type use to test if the reason isn't working is because of sip itself;",
			"typing in some ends of love commands like getFPS without the brackets will also return their data.",
		}
		for i,v in ipairs(temp) do
			table.insert(output,v)
		end
		temp = nil
	elseif data[1] == "print" then
		local temp = {}
		for i = 2,#data do
			table.insert(temp,data[2])
		end
		table.insert(output,table.concat(temp))
		temp = nil
	elseif data[1] == "get" and #data == 2 then
		if _G[data[2]] ~= null and type(_G[data[2]]) ~= "table" then
			if type(_G[data[2]]) == "string" or type(_G[data[2]]) == "number" then
				table.insert(output,data[2].." equals ".._G[data[2]]..".")
			elseif type(_G[data[2]]) == "boolean" then
				if _G[data[2]] then
					table.insert(output,data[2].." equals true.")
				else
					table.insert(output,data[2].." equals false.")
				end
			end
		else
			table.insert(output,"Unrecognised get.")
		end
	elseif data[1] == "set" and #data == 3 then
		if _G[data[2]] ~= null and type(_G[data[2]]) ~= "table" then
			if type(_G[data[2]]) == "string" or type(_G[data[2]]) == "number" then
				_G[data[2]] = data[3]
				table.insert(output,data[2].." now equals ".._G[data[2]]..".")
			elseif type(_G[data[2]]) == "boolean" then
				if data[3] == "true" then
					_G[data[2]] = true
					table.insert(output,data[2].." now equals true.")
				elseif data[3] == "false" then
					_G[data[2]] = false
					table.insert(output,data[2].." now equals false.")
				end
			end
		else
			table.insert(output,"Unrecognised set.")
		end
	elseif data[1] == "length" and #data == 2 then
		if _G[data[2]] ~= null and type(_G[data[2]]) == "table" then
			table.insert(output,"Length is "..#_G[data[2]]..".")
		else
			table.insert(output,"Unrecognised length.")
		end
	
	elseif data[1] == "getFPS" and #data == 1 then
		table.insert(output,love.timer.getFPS())
	elseif data[1] == "getNumSources" and #data == 1 then
		table.insert(output,love.audio.getNumSources())
	elseif data[1] == "getWorkingDirectory" and #data == 1 then
		table.insert(output,love.filesystem.getWorkingDirectory())
	elseif data[1] == "getUserDirectory" and #data == 1 then
		table.insert(output,love.filesystem.getUserDirectory())
	elseif data[1] == "getSaveDirectory" and #data == 1 then
		table.insert(output,love.filesystem.getSaveDirectory())
	else
		table.insert(output,"Unrecognised input.")
	end
	for i = #output,1,-1 do
		table.insert(sip.data.output,output[i])
	end
	data = nil
	output = nil
end