love.graphics.setMode(1024,768)

--[[
-- BOX COLLISIONS

-- Collision detection function.
-- Checks if box1 and box2 overlap.
-- w and h mean width and height.
function checkCollision(box1x, box1y, box1w, box1h, box2x, box2y, box2w, box2h)
    if box1x > box2x + box2w - 1  or -- Is box1 on the right side of box2?
       box1y > box2y + box2h - 1  or -- Is box1 under box2?
       box2x > box1x + box1w - 1  or -- Is box2 on the right side of box1?
       box2y > box1y + box1h - 1     -- Is b2 under b1?
    then
        return false                -- No collision. Yay!
    else
        return true                 -- Yes collision. Ouch!
    end
end]]

function nearestThing(x,y,w,h,t)
	return nearestestCheck(x,y,w,h,t)
end

function nearestestCheck(x,y,w,h,t)
	local id = 0
	for i,v in ipairs(t) do
		if checkCollision(x,y,w,h,v.x,v.y,v.w,v.h) then
			id = i
		end
	end
	return id
end

function aipositions(dt)
	for i,v in ipairs(ai) do
		if v.gene ~= "corpse" and v.gene ~= "plant" then
			v.pos = nearestThing(v.x+v.w/2,v.y+v.h/2,1,1,watertable)
			if v.obj ~= "none" then
				if type(v.obj) == "table" then
					local x,y = velocinator(v.obj[1]-v.x,v.obj[2]-v.y)
					v.x = v.x + ((x*speed)*dt)
					v.y = v.y + ((y*speed)*dt)
					if v.x > v.obj[1]-2 and v.y > v.obj[2]-2 and v.x < v.obj[1]+2 and v.y < v.obj[2]+2 then
						v.obj = "none"
					end
					--local x2,y2 = velocinator(v.obj[1]-v.x,v.obj[2]-v.y)
					--if x ~= x2 or y ~= y2 then
					--	v.obj = "none"
					--end
					--x,y,x2,y2 = nil,nil,nil,nil
					x,y = nil,nil
				elseif type(v.obj) == "number" then
					local x,y = velocinator(ai[v.obj].x-v.x,ai[v.obj].y-v.y)
					v.x = v.x + ((x*speed)*dt)
					v.y = v.y + ((y*speed)*dt)
					if v.x == ai[v.obj].x and v.y == ai[v.obj].y then
						v.obj = "none"
					end
				end
			end
		end
	end
end

function initload()
	watertable = {}
	for i = 1,768/64 do
		for j = 1,1024/64 do
			table.insert(watertable,{x=0+(64*(j-1)),y=0+(64*(i-1)),w=65,h=65,water = math.random(0,100)})
		end
	end
	ai = {}
	for i = 1,15 do
		local temp = {math.random(0,1014),math.random(0,758)}
		table.insert(ai,{gene="plant",state="feed",x=temp[1],w=16,h=16,y=temp[2],gen = 0,pos = nearestThing(temp[1]+16/2,temp[2]+16/2,1,1,watertable),health = 100, needs = {water = 25}})
		temp = nil
	end
	for i = 1,10 do
		local temp = {math.random(0,1014),math.random(0,758)}
		table.insert(ai,{gene="roamer",state="feed",obj="none",x=temp[1],w=10,h=10,y=temp[2],gen = 0,pos = nearestThing(temp[1]+10/2,temp[2]+10/2,1,1,watertable),health = 100, needs = {hunger = 10,loneliness = 0}})
		temp = nil
	end
	datalog = {}
	aipositions()
	heaven = 0
	highestgen = {0,"plant"}
	heaventable = {0}
	tick = 0
	speed = 20
	currentai = {"roamer",1}
	aitypes = {"roamer","scavenger","hunter","plant"}
end

initload()
screenHeight = love.graphics.getHeight() - 2

function aiimpulses(dt)
	for i,v in ipairs(ai) do
		if v.gene ~= "plant" and v.gene ~= "corpse" then
			if v.needs.hunger < 100 then
				v.needs.hunger = v.needs.hunger + (1*dt)
			end
			if v.needs.loneliness < 100 then
				v.needs.loneliness = v.needs.loneliness + (1*dt)
			end
			
			if v.needs.hunger > 90 then
				v.health = v.health - ((v.needs.hunger-90)*dt)
			end	
		elseif v.gene == "plant" then
			if v.needs.water < 100 then
				v.needs.water = v.needs.water + (1*dt)
			end
			if v.needs.water > 90 then
				v.health = v.health - ((v.needs.water-90)*dt)
			end
		end
		if v.health < 0 and v.gene ~= "corpse" then
			v.gene = "corpse"
			v.health = 50
			heaven = heaven + 1
		elseif v.health < 0 and v.gene == "corpse" then
			table.remove(ai,i)
		end
	end
	for i,v in ipairs(watertable) do
		if v.water < 100 then
			v.water = v.water + (1*dt)
		end
	end
end

-- decisionmaking and objective setting
function aiintelligence(dt)
	for i,v in ipairs(ai) do
		if v.gene == "plant" then
			if v.state == "breed" then
				local temp1 = math.random(1,2)
				local temp2 = math.random(1,2)
				if temp1 == 1 then
					temp1 = math.random(-25,0)
				else
					temp1 = math.random(0,25)
				end
				if temp2 == 1 then
					temp2 = math.random(-25,0)
				else
					temp2 = math.random(0,25)
				end
				table.insert(ai,{gene="plant",state="feed",x=v.x+temp1,y=v.y+temp2,w=16,h=16,gen = v.gen+1,pos=nearestThing(v.x+temp1+16/2,v.y+temp2+16/2,1,1,watertable),health = 50, needs = {water = 50}})
				if ai[#ai].gen > highestgen[1] then
					highestgen = {ai[#ai].gen,v.gene}
				end
				if heaven > 0 then
					heaven = heaven - 1
				end
				if ai[#ai].pos == 0 or ai[#ai].pos == nil then
					table.remove(ai)
				end
				v.state = "feed"
				v.needs.water = v.needs.water + 25
				temp1 = nil
				temp2 = nil
			elseif v.state == "feed" then
				if v.health < 100 or v.needs.water > 0 then
					if watertable[v.pos].water > 1 then
						if v.health < 100 then
							watertable[v.pos].water = watertable[v.pos].water - 1
							v.health = v.health + 1
						end
						if v.needs.water > 0 then
							watertable[v.pos].water = watertable[v.pos].water - 1
							v.needs.water = v.needs.water - 5
						end
					end
				elseif v.health >= 100 or v.needs.water <= 0 then
					v.state = "breed"
				end
			end
		elseif v.gene == "roamer" then
			--setting up
			temp = {}
			for j = 1,#ai do
				if j ~= i then
					if ai[j].pos == v.pos then
						table.insert(temp,j)
					end
				end
			end
			-- changing states and adjusting variables
			if v.needs.hunger > 50 and v.state ~= "feed" then
				v.state = "feed"
				v.obj = "none"
			elseif v.needs.hunger <= 0 and v.needs.loneliness > 0 and v.needs.loneliness <= 70 and v.state ~= "breed" then
				v.state = "breed"
				v.obj = "none"
			elseif v.needs.loneliness > 70 and v.state ~= "company" and v.state ~= "feed" then
				v.state = "company"
				v.obj = "none"
			end
			for j,w in ipairs(temp) do
				local temp2 = ai[w].gene
				if temp2 == "hunter" then
					v.state = "flee"
					local temp4 = reverse(velocinator(ai[j].x,ai[j].y))
					local x,y = temp4[1],temp4[2]
					v.obj = {v.x+(x*64),v.y+(y*64)}
					if v.obj[1] < 0 then
						v.obj[1] = 0
					elseif v.obj[1] > 1024 then
						v.obj[1] = 1024
					end
					if v.obj[2] < 0 then
						v.obj[2] = 0
					elseif v.obj[2] > 768 then
						v.obj[2] = 768
					end
					temp4,x,y = nil,nil,nil
				elseif temp2 == "baby" or temp2 == "roamer" then
					if v.needs.loneliness > 0 then
						v.needs.loneliness = v.needs.loneliness - (1*dt)
					end
				end
				-- following instincs if an obj isnt already set and what its looking for is in the area
				if type(v.obj) ~= "number" then
					if v.state == "breed" then
						if temp2 == "roamer" then
							v.obj = w
						end
					elseif v.state == "feed" then
						if temp2 == "plant" then
							v.obj = w
						end
					elseif v.state == "company" then
						if temp2 == "roamer" then
							v.obj = ai[w].obj
						end
					end
				end
				temp2 = nil
			end
			if type(v.obj) == "number" then
				temp2 = ai[v.obj]
				if v.x+v.w/2 < temp2.x+temp2.w + 5 and
					v.x+v.w/2 > temp2.x - 5 and
					v.y+v.h/2 < temp2.y+temp2.h + 5 and
					v.y+v.h/2 > temp2.y - 5 then
					
					if v.state == "feed" then
						if v.health < 100 then
							temp2.health = temp2.health - 1
							v.health = v.health + 1
						end
						if v.needs.hunger > 0 then
							temp2.health = temp2.health - 1
							v.needs.hunger = v.needs.hunger - 5
						end
					elseif v.state == "breed" then
						--v.state = "feed"
						v.obj = "none"
						v.needs.hunger = v.needs.hunger + 20
						v.needs.loneliness = 0
						table.insert(ai,{gene="baby",state="feed",obj="none",x=v.x,w=6,h=6,y=v.y,gen = v.gen + 1,pos = nearestThing(v.x+6/2,v.y+6/2,1,1,watertable),health = 50, needs = {hunger = 0,loneliness = 0}})
						if ai[#ai].gen > highestgen[1] then
							highestgen = {ai[#ai].gen,v.gene}
						end
						if heaven > 0 then
							heaven = heaven - 1
						end
					end
				end
				temp = nil
			elseif v.obj == "none" then
				local temp2 = {math.random(-1,1),math.random(-1,1)}
				temp2 = v.pos + (temp2[1]+(temp2[2]*16))
				if temp2 > 0 and temp2 <= #watertable and temp2 ~= v.pos then
					v.obj = {watertable[temp2].x+math.random(0,64),watertable[temp2].y+math.random(0,64)}
				end
				temp2 = nil
			end
			if v.health < 25 and v.needs.loneliness > 20 then
				v.gene = "scavenger"
				v.state = "feed"
				v.obj = "none"
			end
			temp = nil
		elseif v.gene == "baby" then
			--setting up
			temp = {}
			for j = 1,#ai do
				if j ~= i then
					if ai[j].pos == v.pos then
						table.insert(temp,j)
					end
				end
			end
			-- changing states and adjusting variables
			if v.needs.hunger > 50 and v.state ~= "feed" then
				v.state = "feed"
				v.obj = "none"
			end
			for j,w in ipairs(temp) do
				local temp2 = ai[w].gene
				if temp2 == "hunter" then
					v.state = "flee"
					local temp4 = reverse(velocinator(ai[j].x,ai[j].y))
					local x,y = temp4[1],temp4[2]
					v.obj = {v.x+(x*64),v.y+(y*64)}
					if v.obj[1] < 0 then
						v.obj[1] = 0
					elseif v.obj[1] > 1024 then
						v.obj[1] = 1024
					end
					if v.obj[2] < 0 then
						v.obj[2] = 0
					elseif v.obj[2] > 768 then
						v.obj[2] = 768
					end
					temp4,x,y = nil,nil,nil
				elseif temp2 == "baby" or temp2 == "roamer" then
					if v.needs.loneliness > 0 then
						v.needs.loneliness = v.needs.loneliness - (1*dt)
					end
				end
				-- following instincs if an obj isnt already set and what its looking for is in the area
				if type(v.obj) ~= "number" then
					if v.state == "feed" then
						if temp2 == "plant" then
							v.obj = w
						end
					end
				end
				temp2 = nil
			end
			if type(v.obj) == "number" then
				temp2 = ai[v.obj]
				if v.x+v.w/2 < temp2.x+temp2.w + 5 and
					v.x+v.w/2 > temp2.x - 5 and
					v.y+v.h/2 < temp2.y+temp2.h + 5 and
					v.y+v.h/2 > temp2.y - 5 then
					
					if v.state == "feed" then
						if v.health < 100 then
							temp2.health = temp2.health - 1
							v.health = v.health + 1
						end
						if v.needs.hunger > 0 then
							temp2.health = temp2.health - 1
							v.needs.hunger = v.needs.hunger - 5
						end
					end
				end
				temp = nil
			elseif v.obj == "none" then
				local temp2 = {math.random(-1,1),math.random(-1,1)}
				temp2 = v.pos + (temp2[1]+(temp2[2]*16))
				if temp2 > 0 and temp2 <= #watertable and temp2 ~= v.pos then
					v.obj = {watertable[temp2].x+math.random(0,64),watertable[temp2].y+math.random(0,64)}
				end
				temp2 = nil
			end
			if v.health >= 100 then
				v.gene = "roamer"
				v.w = 10
				v.h = 10
			end
			if v.health < 10 then
				v.gene = "scavenger"
				v.w,v.h = 10,10
				v.state = "feed"
				v.obj = "none"
			end
			temp = nil
		elseif v.gene == "scavenger" then
			--setting up
			temp = {}
			for j = 1,#ai do
				if j ~= i then
					if ai[j].pos == v.pos then
						table.insert(temp,j)
					end
				end
			end
			-- changing states and adjusting variables
			if v.needs.hunger > 50 and v.state ~= "feed" then
				v.state = "feed"
				v.obj = "none"
			elseif v.needs.hunger <= 0 and v.needs.loneliness > 20 and v.state ~= "breed" then
				v.state = "breed"
				v.obj = "none"
			elseif v.needs.loneliness > 70 and v.state ~= "company" and v.state ~= "feed" then
				v.state = "company"
				v.obj = "none"
			end
			for j,w in ipairs(temp) do
				local temp2 = ai[w].gene
				if temp2 == "hunter" then
					v.state = "flee"
					local temp4 = reverse(velocinator(ai[j].x,ai[j].y))
					local x,y = temp4[1],temp4[2]
					v.obj = {v.x+(x*64),v.y+(y*64)}
					if v.obj[1] < 0 then
						v.obj[1] = 0
					elseif v.obj[1] > 1024 then
						v.obj[1] = 1024
					end
					if v.obj[2] < 0 then
						v.obj[2] = 0
					elseif v.obj[2] > 768 then
						v.obj[2] = 768
					end
					temp4,x,y = nil,nil,nil
				elseif temp2 == "scavenger" then
					if v.needs.loneliness > 0 then
						v.needs.loneliness = v.needs.loneliness - (1*dt)
					end
				end
				-- following instincs if an obj isnt already set and what its looking for is in the area
				if type(v.obj) ~= "number" then
					if v.state == "breed" then
						if temp2 == "scavenger" then
							v.obj = w
						end
					elseif v.state == "feed" then
						if temp2 == "corpse" or temp2 == "plant" then
							v.obj = w
						end
					elseif v.state == "company" then
						if temp2 == "scavenger" then
							v.obj = ai[w].obj
						end
					end
				end
				temp2 = nil
			end
			if type(v.obj) == "number" then
				temp2 = ai[v.obj]
				if v.x+v.w/2 < temp2.x+temp2.w + 5 and
					v.x+v.w/2 > temp2.x - 5 and
					v.y+v.h/2 < temp2.y+temp2.h + 5 and
					v.y+v.h/2 > temp2.y - 5 then
					
					if v.state == "feed" then
						if v.health < 100 then
							temp2.health = temp2.health - 1
							v.health = v.health + 1
						end
						if v.needs.hunger > 0 then
							temp2.health = temp2.health - 1
							v.needs.hunger = v.needs.hunger - 5
						end
					elseif v.state == "breed" then
						v.state = "feed"
						v.obj = "none"
						v.needs.hunger = v.needs.hunger + 50
						v.needs.loneliness = 0
						table.insert(ai,{gene="scavenger",state="feed",obj="none",x=v.x,w=6,h=6,y=v.y,gen = v.gen + 1,pos = nearestThing(v.x+6/2,v.y+6/2,1,1,watertable),health = 50, needs = {hunger = 0,loneliness = 0}})
						if ai[#ai].gen > highestgen[1] then
							highestgen = {ai[#ai].gen,v.gene}
						end
						if heaven > 0 then
							heaven = heaven - 1
						end
					end
				end
				temp = nil
			elseif v.obj == "none" then
				local temp2 = {math.random(-1,1),math.random(-1,1)}
				temp2 = v.pos + (temp2[1]+(temp2[2]*16))
				if temp2 > 0 and temp2 <= #watertable and temp2 ~= v.pos then
					v.obj = {watertable[temp2].x+math.random(0,64),watertable[temp2].y+math.random(0,64)}
				end
				temp2 = nil
			end
			if v.health <= 30 and v.needs.loneliness <= 0 then
				v.gene = "hunter"
				v.state = "feed"
				v.health = 40
				v.loneliness = 70
			end
			if v.health >= 100 and v.w == 6 then
				v.w = 10
				v.h = 10
			end
			temp = nil
		elseif v.gene == "hunter" then
			--setting up
			temp = {}
			for j = 1,#ai do
				if j ~= i then
					if ai[j].pos == v.pos then
						table.insert(temp,j)
					end
				end
			end
			-- changing states and adjusting variables
			if v.needs.hunger > 50 and v.state ~= "feed" then
				v.state = "feed"
				v.obj = "none"
			elseif v.needs.hunger <= 0 and v.needs.loneliness > 20 and v.state ~= "breed" then
				v.state = "breed"
				v.obj = "none"
			elseif v.needs.loneliness > 50 and v.state ~= "company" and v.state ~= "feed" then
				v.state = "company"
				v.obj = "none"
			end
			for j,w in ipairs(temp) do
				local temp2 = ai[w].gene
				if temp2 == "hunter" then
					if v.needs.loneliness > 0 then
						v.needs.loneliness = v.needs.loneliness - (1*dt)
					end
				end
				-- following instincs if an obj isnt already set and what its looking for is in the area
				if type(v.obj) ~= "number" then
					if v.state == "breed" then
						if temp2 == "hunter" then
							v.obj = w
						end
					elseif v.state == "feed" then
						if temp2 == "scavanger" or temp2 == "roamer" then
							v.obj = w
						end
					elseif v.state == "company" then
						if temp2 == "hunter" then
							v.obj = ai[w].obj
						end
					end
				end
				temp2 = nil
			end
			if type(v.obj) == "number" then
				temp2 = ai[v.obj]
				if v.x+v.w/2 < temp2.x+temp2.w + 5 and
					v.x+v.w/2 > temp2.x - 5 and
					v.y+v.h/2 < temp2.y+temp2.h + 5 and
					v.y+v.h/2 > temp2.y - 5 then
					
					if v.state == "feed" then
						if v.health < 100 then
							temp2.health = temp2.health - 1
							v.health = v.health + 1
						end
						if v.needs.hunger > 0 then
							temp2.health = temp2.health - 1
							v.needs.hunger = v.needs.hunger - 5
						end
					elseif v.state == "breed" then
						v.state = "feed"
						v.obj = "none"
						v.needs.hunger = v.needs.hunger + 50
						v.needs.loneliness = 0
						table.insert(ai,{gene="hunter",state="feed",obj="none",x=v.x,w=6,h=6,y=v.y,gen = v.gen + 1,pos = nearestThing(v.x+6/2,v.y+6/2,1,1,watertable),health = 50, needs = {hunger = 0,loneliness = 0}})
						if ai[#ai].gen > highestgen[1] then
							highestgen = {ai[#ai].gen,v.gene}
						end
						if heaven > 0 then
							heaven = heaven - 1
						end
					end
				end
				temp = nil
			elseif v.obj == "none" then
				local temp2 = {math.random(-1,1),math.random(-1,1)}
				temp2 = v.pos + (temp2[1]+(temp2[2]*16))
				if temp2 > 0 and temp2 <= #watertable and temp2 ~= v.pos then
					v.obj = {watertable[temp2].x+math.random(0,64),watertable[temp2].y+math.random(0,64)}
				end
				temp2 = nil
			end
			if v.health >= 100 and v.w == 6 then
				v.w = 10
				v.h = 10
			end
			temp = nil
		end
	end
end

function velocinator(x,y)
	if math.abs(x) >= math.abs(y) then
		return x/math.abs(x),y/math.abs(x)
	else
		return x/math.abs(y),y/math.abs(y)
	end
end

function reverse(...)
	local output = {...}
	for i,v in ipairs(output) do
		v = 0 - v
	end
	return output
end

function updater(dt)
	aiimpulses(dt)
	aiintelligence(dt)
	aipositions(dt)
	
	if tick <= 2 then
		tick = tick + (2*dt)
	elseif tick > 2 then
		tick = 0
		if #heaventable > 103 then
			table.remove(heaventable,1)
		end
		table.insert(heaventable,heaven)
	end
end

function largest(t)
	local output = 0
	for i,v in ipairs(t) do
		if v > output then
			output = v
		end
	end
	return output
end

function graphics()
	for i,v in ipairs(watertable) do
		love.graphics.setColor(v.water/2,0,v.water/2)
		love.graphics.rectangle("fill",v.x,v.y,63,63)
	end
	for i,v in ipairs(ai) do
		if v.gene == "corpse" then
			love.graphics.setColor(100,100,100)
		elseif v.gene == "plant" then
			love.graphics.setColor(0,255,0)
		elseif v.gene == "roamer" then
			love.graphics.setColor(0,0,255)
		elseif v.gene == "baby" then
			love.graphics.setColor(0,0,255)
		elseif v.gene == "scavenger" then
			love.graphics.setColor(150,0,200)
		elseif v.gene == "hunter" then
			love.graphics.setColor(255,0,0)
		end
		love.graphics.rectangle("fill",v.x,v.y,v.w,v.h)
	end
	
	love.graphics.print("Heaven: "..heaven..". Fullest heaven on graph: "..largest(heaventable)..". Generation: "..highestgen[1]..","..highestgen[2]..". Move speed: "..speed..". CurrentAI: "..currentai[1],0,0)
	--[[love.graphics.print(ai[16].x..":"..ai[16].y,0,0)
	if ai[16].obj == "none" then
		love.graphics.print("none",0,25)
	end]]
	for i = #heaventable,1,-1 do	
		if i ~= #heaventable then
			love.graphics.line(10*(#heaventable-(i+1)),screenHeight-((heaventable[i+1]/largest(heaventable))*100),10*(#heaventable-i),screenHeight-((heaventable[i]/largest(heaventable))*100))
		end
	end
	if largest(heaventable) == 0 then
		love.graphics.line(0,screenHeight,#heaventable*10,screenHeight)
	end
end

function love.keypressed(key)
	if key == "f1" then
		initload()
	elseif key == "f2" then
		love.graphics.toggleFullscreen()
	elseif key == "escape" then
		info.reset()
	elseif key == "up" then
		speed = speed + 1
	elseif key == "down" then
		if speed > 1 then
			speed = speed -1
		end
	end
end

function love.mousepressed(x,y,button)
	mouseAction(x,y,button)
end

function mouseAction(ix,iy,button)
	if button == "l" then
		if currentai[1] == "plant" then
			table.insert(ai,{gene="plant",state="feed",x=ix,w=16,h=16,y=iy,gen = 0,pos = nearestThing(ix+16/2,iy+16/2,1,1,watertable),health = 100, needs = {water = 25}})
		else
			table.insert(ai,{gene=currentai[1],state="feed",obj="none",x=ix,w=10,h=10,y=iy,gen = 0,pos = nearestThing(ix+10/2,iy+10/2,1,1,watertable),health = 100, needs = {hunger = 10,loneliness = 0}})
		end
	end
	if button == "wu" then
		if currentai[2] ~= #aitypes then
			currentai = {(aitypes[currentai[2]+1]),currentai[2]+1}
		end
	end
	if button == "wd" then
		if currentai[2] ~= 1 then
			currentai = {(aitypes[currentai[2]-1]),currentai[2]-1}
		end
	end
end